@@ -40,6 +40,12 @@
 #' correspondence. The user must supply a list of landmark pairs. A vector containing information 
 #' on individuals must also be supplied. Replicates of each specimen may also be included in the 
 #' dataset, and when specified will be used as measurement error. 
+#' 
+#' The function also provides individual measures of signed and unsigned asymmetry, calculated as the
+#' Procrustes distance between the right and left element (for paired structures, as detailed in 
+#' Klingenberg and McIntyre 1998) or side of the structure (for object symmetry, following Lazić 
+#' et al 2005). The computational difference betwen the two approaches consists in that, for object
+#' symmetry, only paired landmarks are considered, excluding the landmarks of the midline.
 #'  
 #' \subsection{Notes for geomorph 3.0}{ 
 #' Compared to older versions of geomorph, some results can be expected to be slightly different. 
@@ -88,13 +94,22 @@
 #' which might be of interest for advanced users.
 #' @param RRPP A logical value indicating whether residual randomization should be used for 
 #' significance testing.
+#' @param SS.type A choice between type I (sequential), type II (hierarchical), or type III (marginal).
+#' @param turbo A logical value that if TRUE, suppresses coefficient estimation in every random permutation, 
+#' in order to speed up computation time.
+#' @param Parallel Either a logical value to indicate whether parallel processing 
+#' should be used or a numeric value to indicate the number of cores to use in 
+#' parallel processing via the \code{parallel} library. 
+#' If TRUE, this argument invokes forking of all processor cores, except one.  If
+#' FALSE, only one core is used. A numeric value directs the number of cores to use,
+#' but one core will always be spared.
 #' @param print.progress A logical value to indicate whether a progress bar should be printed to 
 #' the screen.  
 #' This is helpful for long-running analyses.
 #' @param ... Arguments to pass onto gpagen
 #' @keywords analysis
 #' @export
-#' @author Dean Adams and Michael Collyer
+#' @author Dean Adams, Michael Collyer and Antigoni Kaliontzopoulou
 #' @return An object of class "bilat.symmetry" returns a list of the following
 #' \item{shape.anova}{An analysis of variance table for the shape data.}
 #' \item{size.anova}{An analysis of variance table for the shape data (when object.sym = FALSE).}
@@ -104,6 +119,10 @@
 #' \item{FA.component}{The fluctuating asymmetry component for each specimen, 
 #' found as the specimen specific side deviation adjusted for the mean 
 #' directional asymmetry in the dataset.}
+#' \item{signed.AI}{Individual signed asymmetry index, as per Klingenberg and McIntyre, 1998;
+#' Lazić et al 2005.}
+#' #' \item{unsigned.AI}{Individual unsigned asymmetry index, as per Klingenberg and McIntyre, 1998;
+#' Lazić et al 2005.}
 #' \item{data.type}{A value indicating whether the analysis was performed as Object or Matching 
 #' symmetry.}
 #' \item{permutations}{The number of random permutations used.}
@@ -122,6 +141,10 @@
 #' bilateral symmetry of shapes. Biometrika. 87:285-300.
 #' @references Klingenberg, C.P., M. Barluenga, and A. Meyer. 2002. Shape analysis of symmetric 
 #' structures: quantifying variation among individuals and asymmetry. Evolution. 56:1909-1920.
+#' @references Lazić, M. M., M. A. Carretero, J. Crnobrnja-Isailović, and A. Kaliontzopoulou. 2015.
+#' Effects of environmental disturbance on phenotypic variation: an integrated assessment of 
+#' canalization, developmental stability, modularity, and allometry in lizard head shape. The American
+#' Naturalist 185:44–58.
 #' @examples
 #' #Example of matching symmetry
 #'
@@ -170,41 +193,42 @@
 #' # NOTE one can also: scallop.sym$data.type # recall the symmetry type
 
 bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.sym = FALSE, land.pairs = NULL,
-                         data = NULL, iter = 999, seed = NULL, RRPP = TRUE, print.progress = TRUE, ...){
-
+                           data = NULL, iter = 999, seed = NULL, RRPP = TRUE, SS.type = c("I", "II", "III"),
+                           turbo = TRUE, Parallel = FALSE, print.progress = TRUE, ...){
+  
   if(!is.null(data)){
     data <- droplevels(data)
     A.name <- deparse(substitute(A))
     A.name.match <- match(A.name, names(data))[1]
     if(is.na(A.name.match)) A.name.match <- NULL
-    } else A.name.match <- NULL
+  } else A.name.match <- NULL
   
-    if(is.null(A.name.match) && is.gpagen(A)) {
+  if(is.null(A.name.match) && is.gpagen(A)) {
     size <- A$Csize
     A <- A$coords
-    } else if(is.null(A.name.match) && inherits(A, "geomorphShapes")) {
-      A <- gpagen(A, ...)
+  } else if(is.null(A.name.match) && inherits(A, "geomorphShapes")) {
+    A <- gpagen(A, ...)
+    size <- A$Csize
+    A <- A$coords
+  } else {
+    if(!is.null(data)) {
+      if(is.null(A.name.match)) stop("Coordinates are not part of the data frame provided")
+      A <- data[[A.name.match]]
+      if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').") 
+      if (length(dim(A))!=3) stop("Data matrix not a 3D array (see 'arrayspecs').") 
+      if(print.progress) cat("\nInitial GPA\n")
+      A <- gpagen(A, print.progress = print.progress, ...)
       size <- A$Csize
       A <- A$coords
     } else {
-      if(!is.null(data)) {
-        if(is.null(A.name.match)) stop("Coordinates are not part of the data frame provided")
-          A <- data[[A.name.match]]
-          if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').") 
-          if (length(dim(A))!=3) stop("Data matrix not a 3D array (see 'arrayspecs').") 
-          if(print.progress) cat("\nInitial GPA\n")
-          A <- gpagen(A, print.progress = print.progress, ...)
-          size <- A$Csize
-          A <- A$coords
-        } else {
-          if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').") 
-          if (length(dim(A))!=3) stop("Data matrix not a 3D array (see 'arrayspecs').") 
-          if(print.progress) cat("\nInitial GPA\n")
-          A <- gpagen(A, print.progress = print.progress, ...)
-          size <- A$Csize
-          A <- A$coords
-        }
+      if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').") 
+      if (length(dim(A))!=3) stop("Data matrix not a 3D array (see 'arrayspecs').") 
+      if(print.progress) cat("\nInitial GPA\n")
+      A <- gpagen(A, print.progress = print.progress, ...)
+      size <- A$Csize
+      A <- A$coords
     }
+  }
   
   if(is.null(data)){
     if(is.null(ind)) stop("Individuals not specified.") else ind <- factor(ind, levels = unique(ind))
@@ -214,7 +238,7 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     ind.match <- match(names(data), "ind")
     if(all(is.na(ind.match))) stop("Individuals not specified in geomorph data frame")
     ind <- factor(data[[which(!is.na(ind.match))]])
-       ind <- factor(ind, levels = unique(ind))  
+    ind <- factor(ind, levels = unique(ind))  
     side.match <- match(names(data), "side")
     if(all(is.na(side.match))) side <- NULL else 
       side <- factor(data[[which(!is.na(side.match))]])
@@ -222,7 +246,7 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     if(all(is.na(replicate.match))) replicate <- NULL else 
       replicate <- factor(data[[which(!is.na(replicate.match))]])
   }
-    
+  
   n <- dim(A)[3]; k <- dim(A)[2]; p <- dim(A)[1]; nind <- nlevels(ind); spec.names <- dimnames(A)[[3]]
   if(!object.sym && is.null(side)) stop("Sides not specified.") 
   
@@ -239,21 +263,22 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     gpa.res <- gpagen(A, print.progress = print.progress)
     A <- gpa.res$coords
   }
-    Y <- two.d.array(A)
-    
-    if(!is.null(replicate)) {
+  Y <- two.d.array(A)
+  
+  if(!is.null(replicate)) {
     form.shape <- Y ~ ind + side + ind/side 
     form.names <- c("ind", "side", "ind:side", "ind:side:replicate", "Total")
     dat.shape <- geomorph.data.frame(Y = Y, ind = ind, side = side, replicate = replicate)
-    } else {
+  } else {
     form.shape <- Y ~ ind + side 
     form.names <- c("ind", "side", "ind:side", "Total")
     dat.shape <- geomorph.data.frame(Y = Y, ind = ind, side = side)
   }
   if(print.progress) cat("\nShape Analysis")
-  PSh <- procD.lm(form.shape, data = dat.shape, RRPP = RRPP, 
-           seed = seed, iter = iter, print.progress = print.progress, 
-           effect.type = "F")
+  PSh <- procD.lm(form.shape, data = dat.shape, RRPP = RRPP, SS.type = SS.type,
+                  Parallel = Parallel, turbo = turbo,
+                  seed = seed, iter = iter, print.progress = print.progress, 
+                  effect.type = "F")
   shape.anova <- anova(PSh, print.progress = FALSE, effect.type = "F")$table
   shape.anova$Z[is.nan(shape.anova$Z)] <- NA
   
@@ -269,15 +294,15 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     random.shape.F[2,] <- MS[2,]/MS[3,]
     random.shape.F[3,] <- PSh$ANOVA$Fs[3,]
     
-    newZ <- apply(log(random.shape.F + 0.000001), 1, effect.size)
-    newP <- apply(random.shape.F , 1, pval)
+    newZ <- apply(random.shape.F, 1, effect.size)
+    newP <- apply(random.shape.F, 1, pval)
     shape.anova$F[1:3] <- random.shape.F[1:3, 1]
     shape.anova$Z[1:3] <- newZ
     shape.anova[[ncol(shape.anova)]][1:3] <- newP
   }
   rownames(shape.anova) <- form.names
-
-
+  
+  
   if(!object.sym){  
     if(!is.null(replicate)) {
       form.size <- size ~ ind + side + ind/side 
@@ -288,6 +313,8 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     }
     if(print.progress) cat("\nSize Analysis")
     PSz <- procD.lm(form.size, data = dat.size, RRPP = RRPP, 
+                    SS.type = SS.type,
+                    Parallel = Parallel, turbo = turbo,
                     seed = seed, iter= iter, print.progress = print.progress, 
                     effect.type = "F")
     size.anova <- anova(PSz, print.progress = FALSE, effect.type = "F")$table
@@ -297,14 +324,14 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
     RSS <- PSz$ANOVA$RSS
     MSE <- RSS / matrix(PSz$ANOVA$df[nrow(RSS) + 1], nrow(RSS), ncol(RSS))
     random.size.F <- MS/MSE
-
+    
     if(length(form.names) > 4) {
       MS <- random.size.F <- PSz$ANOVA$MS
       MS.mod <- PSz$ANOVA$RSS.model[3,]/PSz$ANOVA$df[4]
       random.size.F[1,] <- MS[1,]/MS[3,]
       random.size.F[2,] <- MS[2,]/MS[3,]
       random.size.F[3,] <- PSz$ANOVA$Fs[3,]
-      newZ <- apply(log(random.size.F + 0.000001), 1, effect.size)
+      newZ <- apply(random.size.F, 1, effect.size)
       newP <- apply(random.size.F , 1, pval)
       size.anova$F[1:3] <- random.size.F[1:3, 1]
       size.anova$Z[1:3] <- newZ
@@ -327,14 +354,15 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
   indsq <- seq(n.side, (n.ind*n.side), n.side)
   asymm.component <- avg.side.symm[indsq,] - avg.side.symm[-indsq,]
   mn.shape <- mshape(A)
-  asymm.component <- simplify2array(lapply(1:n.ind, function(j) 
-  {t(matrix(asymm.component[j,],k,p)) + mn.shape}))
+  asymm.component <- simplify2array(lapply(1:n.ind, function(j) {
+    t(matrix(asymm.component[j,],k,p)) + mn.shape
+    }))
   dimnames(asymm.component)[[3]] <- ind.names
   
   DA.est <- coef(.lm.fit(X.side, Y))
   DA.mns <- arrayspecs(rbind(apply(DA.est[-indsq,], 2, mean), apply(DA.est[indsq,], 2, mean)), p, k)
   mn.DA <- matrix(apply((DA.est[-indsq,] - DA.est[indsq,]), 2, mean), byrow=T, nrow=p, ncol=k)
-
+  
   X.ind.side <- model.matrix(~(side:ind) + 0, data = as.data.frame(dat.shape[-1]))
   ind.mns <- coef(.lm.fit(X.ind.side, Y))
   FA.component <- ind.mns[-indsq,] - ind.mns[indsq,]
@@ -342,13 +370,27 @@ bilat.symmetry <- function(A, ind = NULL, side = NULL, replicate = NULL, object.
   {t(matrix(FA.component[j,],k,p)) + mn.shape - mn.DA}))
   dimnames(FA.component)[[3]] <- ind.names 
   
+  # Calculate individual asymmetry indices
+  signed.asymm <- two.d.array(asymm.component)
+  signed.AI <- sqrt(apply(signed.asymm^2, 1, sum))
+  names(signed.AI) <- ind.names
+  
+  asymm.mean <- apply(signed.asymm, 2, mean)
+  unsigned.asymm <- matrix(NA, nrow=nrow(signed.asymm), ncol=ncol(signed.asymm))
+  for (i in 1:nrow(signed.asymm)){
+    unsigned.asymm[i,] <- ifelse(signed.asymm[i,]%*%asymm.mean > 0, signed.asymm[i,], signed.asymm[i,]*(-1))
+  } 
+  unsigned.AI <- sqrt(apply(unsigned.asymm^2, 1, sum))
+  names(unsigned.AI) <- ind.names
+
   out <- list(shape.anova = shape.anova, symm.shape = symm.component,
-            asymm.shape = asymm.component, DA.component = DA.mns, FA.component = FA.component,
-            data.type = ifelse(object.sym == TRUE, "Object", "Matching"),
-            permutations = iter+1,
-            random.shape.F = random.shape.F, 
-            perm.method = ifelse(RRPP==TRUE,"RRPP", "Raw"),
-            procD.lm.shape = PSh)
+              asymm.shape = asymm.component, DA.component = DA.mns, FA.component = FA.component,
+              signed.AI = signed.AI, unsigned.AI = unsigned.AI,
+              data.type = ifelse(object.sym == TRUE, "Object", "Matching"),
+              permutations = iter+1,
+              random.shape.F = random.shape.F, 
+              perm.method = ifelse(RRPP==TRUE,"RRPP", "Raw"),
+              procD.lm.shape = PSh)
   if(!object.sym) {
     out$size.anova = size.anova
     out$procD.lm.size = PSz
