@@ -103,16 +103,19 @@
 #' data(plethspecies)
 #' Y.gpa<-gpagen(plethspecies$land)    #GPA-alignment
 #' gdf <- geomorph.data.frame(Y.gpa, phy = plethspecies$phy)
-#' pleth.pgls <- procD.pgls(coords ~ Csize, phy = phy, data = gdf, iter = 999)
+#' pleth.pgls <- procD.pgls(coords ~ Csize, phy = phy, data = gdf, 
+#' iter = 999)
 #' anova(pleth.pgls)
 #' summary(pleth.pgls)  #similar output
 #' 
 #' ### Working with procD.pgls objects
 #' predict(pleth.pgls)
-#' plot(pleth.pgls, type="regression", reg.type="RegScore", predictor = gdf$Csize)
+#' plot(pleth.pgls, type="regression", reg.type="RegScore", 
+#' predictor = gdf$Csize)
 #' attributes(pleth.pgls) # Note the PGLS object
 #' attributes(pleth.pgls$PGLS) # PGLS details embedded within PGLS object
-#' pleth.pgls$LM$Pcov # the projection matrix derived from the phylogenetic covariance matrix
+#' pleth.pgls$LM$Pcov # the projection matrix derived from the 
+#' # phylogenetic covariance matrix
 #' pleth.pgls$pgls.fitted # the PGLS fitted values 
 #' pleth.pgls$GM$pgls.fitted # The same fitted values, in a 3D array
 procD.pgls<-function(f1, phy, Cov = NULL, iter=999, seed=NULL, int.first = FALSE, 
@@ -139,6 +142,8 @@ procD.pgls<-function(f1, phy, Cov = NULL, iter=999, seed=NULL, int.first = FALSE
                    data = data, print.progress = print.progress,
            ...)
   
+  pgls$call[[2]] <- f1
+  
   names(pgls) <- gsub("gls", "pgls", x = names(pgls))
   if(!is.null(pgls$GM)) names(pgls$GM) <- gsub("gls", "pgls", x = names(pgls$GM))
   
