@@ -50,17 +50,21 @@
 #'plotRefToTarget(M, preds$pred2)
 #'
 #'PC <- PCA$x[,1:2]
-#'# user-picked spots - can be anything, but it in this case, apparent groups
+#'# user-picked spots can be anything, but it in this case, apparent groups
 #'preds <- shape.predictor(Y.gpa$coords, x= PC, Intercept = FALSE, 
-#'                         pred1 = c(0.045,-0.02), pred2 = c(-0.025,0.06), pred3 = c(-0.06,-0.04)) 
+#'                         pred1 = c(0.045,-0.02), 
+#'                         pred2 = c(-0.025,0.06), 
+#'                         pred3 = c(-0.06,-0.04)) 
 #'plotRefToTarget(M, preds$pred1)
 #'plotRefToTarget(M, preds$pred2)
 #'plotRefToTarget(M, preds$pred3)
 #'
 #'# allometry example - straight-up allometry
 #'
-#'preds <- shape.predictor(Y.gpa$coords, x= log(Y.gpa$Csize), Intercept = TRUE, 
-#'                         predmin = min(log(Y.gpa$Csize)), predmax = max(log(Y.gpa$Csize))) 
+#'preds <- shape.predictor(Y.gpa$coords, x= log(Y.gpa$Csize), 
+#'                         Intercept = TRUE, 
+#'                         predmin = min(log(Y.gpa$Csize)), 
+#'                         predmax = max(log(Y.gpa$Csize))) 
 #'
 #'plotRefToTarget(M, preds$predmin, mag=3)
 #'plotRefToTarget(M, preds$predmax, mag=3)
@@ -74,7 +78,8 @@
 #'predictor = log(gdf$Csize),
 #'reg.type ="RegScore") # make sure to have a predictor 
 #'
-#'preds <- shape.predictor(plethAllometry$GM$fitted, x= allom.plot$RegScore, Intercept = FALSE, 
+#'preds <- shape.predictor(plethAllometry$GM$fitted, 
+#'                         x= allom.plot$RegScore, Intercept = FALSE, 
 #'                         predmin = min(allom.plot$RegScore), 
 #'                         predmax = max(allom.plot$RegScore)) 
 #'plotRefToTarget(M, preds$predmin, mag=3)
@@ -84,7 +89,8 @@
 #'type = "regression", 
 #'predictor = log(gdf$Csize),
 #'reg.type ="PredLine")
-#'preds <- shape.predictor(plethAllometry$GM$fitted, x= allom.plot$PredLine, Intercept = FALSE, 
+#'preds <- shape.predictor(plethAllometry$GM$fitted, 
+#'                         x= allom.plot$PredLine, Intercept = FALSE, 
 #'                         predmin = min(allom.plot$PredLine), 
 #'                         predmax = max(allom.plot$PredLine)) 
 #'plotRefToTarget(M, preds$predmin, mag=3)
@@ -92,7 +98,8 @@
 #'
 #'# using factors via PCA
 #'
-#'gdf <- geomorph.data.frame(Y.gpa, species = plethodon$species, site = plethodon$site)
+#'gdf <- geomorph.data.frame(Y.gpa, species = plethodon$species, 
+#'        site = plethodon$site)
 #'pleth <- procD.lm(coords ~ species*site, data=gdf)
 #'PCA <- prcomp(pleth$fitted)
 #'plot(PCA$x, asp=1, pch=19)
@@ -118,7 +125,8 @@
 #'X <- X[,-1]
 #'symJord <- c(0,1,0) # design for P. Jordani in sympatry
 #'alloJord <- c(0,0,0) # design for P. Jordani in allopatry
-#'preds <- shape.predictor(arrayspecs(pleth$fitted, 12,2), x = X, Intercept = TRUE, 
+#'preds <- shape.predictor(arrayspecs(pleth$fitted, 12,2), x = X, 
+#'                         Intercept = TRUE, 
 #'                         symJord=symJord, alloJord=alloJord)
 #'plotRefToTarget(M, preds$symJord, mag=2)
 #'plotRefToTarget(M, preds$alloJord, mag=2)
@@ -133,15 +141,20 @@
 #'summary(PLS)
 #'plot(PLS)
 #'
-#'preds <- shape.predictor(Y.gpa$coords, plethShapeFood$food, Intercept = FALSE,
+#'preds <- shape.predictor(Y.gpa$coords, plethShapeFood$food, 
+#'                         Intercept = FALSE,
 #'                         method = "PLS",
-#'                         pred1 = 2, pred2 = -4, pred3 = 2.5) # using PLS plot as a guide
+#'                         pred1 = 2, pred2 = -4, pred3 = 2.5) 
+#'                         # using PLS plot as a guide
 #'M <- mshape(Y.gpa$coords)
 #'plotRefToTarget(M, preds$pred1, mag=2)
 #'plotRefToTarget(M, preds$pred2, mag=2)
 #'plotRefToTarget(M, preds$pred3, mag=2)
 #'
-shape.predictor <- function(A, x = NULL, Intercept = FALSE, method = c("LS", "PLS"), ...){
+shape.predictor <- function(A, 
+                            x = NULL, 
+                            Intercept = FALSE, 
+                            method = c("LS", "PLS"), ...){
   if(length(dim(A)) != 3) stop("Shape data must be in the form of a p x k x n array")
   dims <- dim(A); p <- dims[1]; k <- dims[2]; n <- dims[3]
   Y <- two.d.array(A)
