@@ -24,8 +24,8 @@
 #'  \subsection{For more than two blocks}{ 
 #' If one wishes to consider 3+ arrays or matrices, there are multiple options.  First, one could perform multiple two.b.pls analyses and use
 #' \code{\link{compare.pls}} to ascertain which blocks are more "integrated".  Second, one could use \code{\link{integration.test}} and perform a test that
-#' averages the amount of integration (correlations) across multiple pairwise blocks.  Note that \code{\link{integration.test}} performed on two matrices or
-#' arrays returns the same results as two.b.pls.  (Thus, \code{\link{integration.test}} is more flexible and thorough.)
+#' averages the amount of integration (correlations) across multiple pairwise blocks.  Note that performing \code{\link{integration.test}} performed on two matrices or
+#' arrays returns the same results as \code{\link{two.b.pls}}.  Thus, \code{\link{integration.test}} is more flexible and thorough.
 #' }
 #' 
 #'  \subsection{Using phylogenies and PGLS}{ 
@@ -91,8 +91,8 @@
 #' summary(PLS)
 #' plot(PLS)
 #'  
-#'  ### Visualize shape variation using picknplot.shape Because picknplot requires 
-#'  ### user decisions, the following example
+#'  ### Visualize shape variation using picknplot.shape Because picknplot  
+#'  ### requires user decisions, the following example
 #'  ### is not run (but can be with removal of #).
 #'  ### For detailed options, see the picknplot help file
 #'  # picknplot.shape(plot(PLS))
@@ -100,23 +100,40 @@
 #' 
 
 two.b.pls <- function (A1, A2,  iter = 999, seed = NULL, print.progress=TRUE){
-    if (any(is.na(A1)) == T) 
-      stop("Data matrix 1 contains missing values. Estimate these first (see 'estimate.missing').")
-    if (any(is.na(A2)) == T) 
-      stop("Data matrix 2 contains missing values. Estimate these first (see 'estimate.missing').")
-  if (is.null(dim(A1))) A1 <- as.matrix(A1); if (is.null(dim(A2))) A2 <- as.matrix(A2)
-  if (length(dim(A1)) == 3) x <- two.d.array(A1) else x <- as.matrix(A1)
-  if (length(dim(A2)) == 3) y <- two.d.array(A2) else y <- as.matrix(A2)
-  if (nrow(x) != nrow(y)) stop("Data matrices have different numbers of specimens.")
+    if (any(is.na(A1))) 
+      stop("\nData matrix 1 contains missing values. Estimate these first (see 'estimate.missing').",
+           call. = FALSE)
+    if (any(is.na(A2))) 
+      stop("\nData matrix 2 contains missing values. Estimate these first (see 'estimate.missing').",
+           call. = FALSE)
+  
+  x <- try(two.d.array(A1), silent = TRUE)
+  if(inherits(x, "try-error")) x <- try(as.matrix(A1), silent = TRUE)
+  if(inherits(x, "try-error"))
+    stop("\nA is not a suitable data array for analysis. ", call. = FALSE)
+  
+  y <- try(two.d.array(A2), silent = TRUE)
+  if(inherits(y, "try-error")) y <- try(as.matrix(A2), silent = TRUE)
+  if(inherits(y, "try-error"))
+    stop("\nA is not a suitable data array for analysis. ", call. = FALSE)
+  
+  x <- as.matrix(x)
+  y <- as.matrix(y)
   n <- nrow(x)
+  namesX <- rownames(x)
+  namesY <- rownames(y)
+  if(is.null(namesX) || is.null(namesY))
+    cat("Data in either A1 or A2 do not have names.  It is assumed data in both A1 and A2 are ordered the same.\n")
   
-  if (is.null(rownames(x))) rownames(x) <- 1:n
-  if (is.null(rownames(y))) rownames(y) <- 1:n
+  if (is.null(namesX)) namesX <- 1:NROW(x)
+  if (is.null(namesY)) namesY <- namesX
   
-  if(length(na.omit(match(rownames(x), rownames(y)))) != n) 
-    stop("Mismatched specimen names for A1 and A2.\n", call. = FALSE)
+  if (length(namesX) != length(namesY)) stop("\nData matrices have different numbers of specimens.",
+                               call. = FALSE)
+  if(length(unique(c(namesX, namesY))) != n) 
+    stop("\nMismatched specimen names for A1 and A2.\n", call. = FALSE)
   
-  y <- y[rownames(x), ]
+  y <- y[match(namesX, namesY), ]
 
   pls.obs <- pls(x, y, RV=FALSE, verbose=TRUE)
   
@@ -130,10 +147,28 @@ two.b.pls <- function (A1, A2,  iter = 999, seed = NULL, print.progress=TRUE){
     d <- which(zapsmall(pcay$sdev) > 0)
     y <- pcay$x[,d]
   }
-  if(print.progress) pls.rand <- apply.pls(center(x), center(y), RV=FALSE, iter=iter, seed=seed) else
-    pls.rand <- .apply.pls(center(x), center(y), RV=FALSE, iter=iter, seed=seed) 
+  
+  if(!is.null(seed) && seed == "random") seed = sample(1:iter, 1)
+  ind <- perm.index(n, iter, seed = seed)
+  perms <- length(ind)
+  
+  if(print.progress){
+    cat(paste("\nRandom PLS calculations:", perms, "permutations.\n"))
+    pb <- txtProgressBar(min = 0, max = perms+1, initial = 0, style=3)
+  }
+  
+  xc <- as.matrix(center(x))
+  yc <- as.matrix(center(y))
+  pls.rand <- sapply(1:perms, function(j) {
+    step <- j
+    if(print.progress) setTxtProgressBar(pb,step)
+    s <- ind[[j]]
+    xs <- as.matrix(xc[s,])
+    quick.pls(xs, yc)
+  })
+
   p.val <- pval(abs(pls.rand))
-  Z <- effect.size(abs(pls.rand), center=TRUE) 
+  Z <- effect.size(pls.rand, center=TRUE) 
   XScores <- pls.obs$XScores
   YScores <- pls.obs$YScores
   out <- list(r.pls = pls.rand[1], P.value = p.val, Z = Z,
